import "Turbine.UI";
import "Turbine.Gameplay";

local C_FRAMEBACK_PATH = "Serbryt/BandAide/Resources/BarBase2.tga"
local C_LINE_X_OFFSET = 30
local C_LINE_Y_OFFSET = 9
local currentIconHeight = 16

MainFrame = class(Turbine.UI.Control);

function DoBarsGoDown()
    return (settings.bargrowmode == 2) or (settings.bargrowmode == 0 and settings.mainwindow.position.top < 0.5);
end

function GetIconSize()
    if currentIconHeight < 16 or currentIconHeight > 32 then
        currentIconHeight = 16;
    end
    return currentIconHeight;
end

function MainFrame:Constructor(parent,x,y,w,h)
    Turbine.UI.Control.Constructor(self);

    self.bestSong = "";
    self.currentSong = nil;
    self.members = { };
    self.currentLength = 0;

    self.player = "Player";
    local localPlayer = Turbine.Gameplay.LocalPlayer.GetInstance();
    if player ~= nil then
        player = player.GetName();
    end

    self:SetBackColor(Turbine.UI.Color(0,0,0,0));
    self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self:SetParent(parent);
    self:SetPosition(x,y);
    self:SetMouseVisible(false);
    self:SetSize(w,h);
    self:SetOpacity(0.5);
    self:SetZOrder(parent:GetZOrder());

    self.playerFrame = PlayerFrame(self,0,0,w,h,self:GetImageBackground());

    self.barFrame = Turbine.UI.Control();
    self.barFrame:SetBackColor(Turbine.UI.Color(0,0,0,0));
    self.barFrame:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.barFrame:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.barFrame:SetParent(parent);
    self.barFrame:SetPosition(0,0);
    self.barFrame:SetMouseVisible(false);
    self.barFrame:SetSize(PlayerBar:GetTotalWidth(),0);
    self.barFrame:SetOpacity(0.5);
    self.barFrame:SetZOrder(parent:GetZOrder()-1);

    self:Show(true);
end

function MainFrame:GetImageBackground()
    return C_FRAMEBACK_PATH;
end

function MainFrame:Show(show)
    self:SetVisible(show);
    self.playerFrame:Show(show);
    self:RecalcPositionFromBarUpdate();
end

function MainFrame:Reset()
    self.bestSong = nil;
    self.currentSong = nil;
    self.currentLength = 0;
    for member, bar in pairs(self.members) do
        bar:RemoveBar()
        self.members[member] = nil;
    end

    self:RecalcPositionFromBarUpdate();
    self:UpdateTitle();
    self.playerFrame:UpdatePlayingTitle(nil, 0);
end

function MainFrame:UpdateTitle()
    local songs = {};
    local lengths = {};
    local numSyncs = 0;
    local numSongs = 0;

    for member, bar in pairs(self.members) do
        numSyncs = numSyncs + 1;
        local song = bar:GetSong();
        local length = bar:GetLength();
        if songs[song] ~= nil then
            songs[song] = songs[song] + 1;
        else
            numSongs = numSongs + 1;
            songs[song] = 1;
        end
        if lengths[length] ~= nil then
            lengths[length] = lengths[length] + 1;
        else
            lengths[length] = 1;
        end
    end

    local bestCount = 0;
    local bestLength = 0;
    self.bestSong = "";

    for song, count in pairs(songs) do
        if count > bestCount then
            bestCount = count;
            self.bestSong = song;
        end
    end

    bestCount = 0;
    for length, count in pairs(lengths) do
        if count > bestCount then
            bestCount = count;
            bestLength = length;
        end
    end

    local mismatch = false;
    for member, bar in pairs(self.members) do
        if bar:IsSongMismatched(self.bestSong, bestLength) == true then
            mismatch = true;
        end
    end

    self.playerFrame:UpdateTitle(numSyncs, mismatch, self.bestSong);
end

function MainFrame:GetNumMembers()
    local numMembers = 0;
    for member, bar in pairs(self.members) do numMembers = numMembers + 1; end
    return numMembers;
end

function MainFrame:AddSync(member, song, instrument, length)
    local bar = self.members[member];

    if bar == nil then
        bar = PlayerBar(self.barFrame, 0, (self:GetNumMembers() * GetIconSize()), self:GetWidth(), GetIconSize(), member, song, instrument, length);
        self.members[member] = bar;
        self:RecalcPositionFromBarUpdate();
    else
        local resort = (bar:GetInstrument() ~= instrument);
        bar:UpdateBar(song, instrument, length);
        if resort == true then
            self:RecalcPositionFromBarUpdate();
        end
    end

    self:UpdateTitle();
end

function MainFrame:RecalcPositionFromBarUpdate()
    local totalMembers = self:GetNumMembers();
    local deltaY = 16;
    if settings.iconsizemode then
        if settings.iconsizemode == 1 then
            deltaY = 16;
        elseif settings.iconsizemode == 2 then
            deltaY = 32;
        else
            deltaY = 32;
            local bigSize = (totalMembers * 32);
            local parent = self:GetParent();
            local tmpX, tmpY = parent:GetAnchorPosition();
            if DoBarsGoDown() or not settings.playerbars then
                if (tmpY+self:GetHeight()+bigSize) > (Turbine.UI.Display:GetHeight()-50) then
                    deltaY=16;
                end
            else
                if (tmpY-bigSize) < 50 then
                    deltaY=16;
                end
            end
        end
    end

    currentIconHeight = deltaY;

    local y = 0;
    for member, bar in spairs(self.members, function(t,a,b) return t[b].icode > t[a].icode end) do
        bar:Reposition(0,y);
        y = y + deltaY;
    end

    local barHeight = (deltaY * totalMembers);
    local newHeight = barHeight + self.playerFrame:GetHeight();
    self:SetHeight(newHeight);
    self.barFrame:SetHeight(barHeight)

    local parent = self:GetParent();
    local x, y = parent:GetAnchorPosition();
    parent:SetHeight(newHeight);

    if DoBarsGoDown() or not settings.playerbars then
        self.barFrame:SetPosition(28,self:GetHeight()-barHeight-28);
        self.playerFrame:SetPosition(0,0);
        parent:SetPosition(x,y);
    else
        self.barFrame:SetPosition(28,4);
        self.playerFrame:SetPosition(0, self:GetHeight()-self.playerFrame:GetHeight());
        parent:SetPosition(x,math.max(30,y-barHeight));
    end
    self.barFrame:SetVisible(self:IsVisible() and settings.playerbars);
end

function MainFrame:LostSync(member)
    local bar = self.members[member];

    if bar ~= nil then
        bar:RemoveBar();
        self.members[member] = nil;
        self:RecalcPositionFromBarUpdate();
        self:UpdateTitle();
    end
end

function MainFrame:OnMusicStarting()
    self.currentSong = Capitalize(self.bestSong);
    self.currentLength = 0;
    if self.currentSong then
        local lengths = {};

        for member, bar in pairs(self.members) do
            local length = bar:GetLength();
            if lengths[length] then
                lengths[length] = lengths[length] + 1;
            else
                lengths[length] = 1;
            end
        end

        local bestCount = 0;

        for length, count in pairs(lengths) do
            if count > bestCount then
                bestCount = count;
                self.currentLength = length;
            end
        end
    end

    self.bestSong = nil;
    for member, bar in pairs(self.members) do
        bar:RemoveBar()
        self.members[member] = nil;
    end

    self:RecalcPositionFromBarUpdate();
    self:UpdateTitle();
    self.playerFrame:UpdatePlayingTitle(self.currentSong, self.currentLength);
    self.currentSong = nil;
end
