import "Turbine.Gameplay";

local C_MOVEGEM_PATH = "Serbryt/BandAide/Resources/MoveGem.tga"
local C_MOVEBAR_PATH = "Serbryt/BandAide/Resources/MoveBar.tga"

MainWindow = class(Turbine.UI.Window);

function MainWindow:Constructor()
    Turbine.UI.Window.Constructor(self);

    self.loaded = false;
    self.layout = Turbine.UI.HorizontalLayout.LeftToRight;
    self.dragable = false;
    self.dragging = false;
    self.hudvisible = true;
    self.options = nil;
    self.shown = false;

    self.KeyDown = function(sender, args)
        if (args.Action == C_ACTION_LOTRO_HUD_TOGGLE) then -- HUD toggle
            self.hudvisible = not self.hudvisible;
            self:CheckVisible();
        elseif (args.Action == C_ACTION_LOTRO_UI_REPOSITION) then -- UI reposition
            self:ToggleRepositioning();
            self:CheckVisible();
        end
    end

    self:SetWantsKeyEvents(false);

    self.Update = function()
        if not self.loaded then
            self.loaded = true;
            self.UpgradeSettings();
            self.shown = settings.firstload or not settings.hideuntilneeded;
            Plugins[C_PLUGIN].Unload = unloadCallback;
            self:SetWantsUpdates(false);

            local testCtl=Turbine.UI.Control();
            testCtl:SetParent(self);
            testCtl:SetBackground(MainFrame:GetImageBackground());
            testCtl:SetStretchMode(2);

            self.baseWidth , self.baseHeight=testCtl:GetSize();
            testCtl:SetVisible(false);

            self.baseWidth = math.max(self.baseWidth, PlayerBar:GetTotalWidth()+10);
            settings.mainwindow.position.left = math.max(0.05, math.min(0.95, settings.mainwindow.position.left));
            settings.mainwindow.position.top = math.max(0.05, math.min(0.95, settings.mainwindow.position.top));

            local x,y = self:GetAnchorPosition();

            self:SetPosition(x,y);
            self:SetSize(self.baseWidth, self.baseHeight);
            self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0 ));
            self:SetZOrder(10);

            self.activeFrame = MainFrame(self,0,0, self:GetWidth(), self:GetHeight());
            self:CheckVisible();
            self:SetWantsKeyEvents(true);
        end
    end

    self:SetWantsUpdates(true);

end

function MainWindow:UpgradeSettings()
    if type(settings.optionswindow.position.left) ~= "number" then
        settings = defaultSettings;
    end

    if settings.version ~=nil and settings.version == C_VERSION then
        return;
    end

    local oldSettings = settings;
    settings = defaultSettings;
    if oldSettings.mainwindow ~= nil then
        settings.mainwindow = oldSettings.mainwindow;
    end
    if oldSettings.optionswindow ~= nil then
        settings.optionswindow = oldSettings.optionswindow;
    end
    if oldSettings.hideuntilneeded ~= nil then
        settings.hideuntilneeded = oldSettings.hideuntilneeded;
    end
    if oldSettings.playerbars ~= nil then
        settings.playerbars = oldSettings.playerbars;
    end
    if oldSettings.bargrowmode ~= nil then
        settings.bargrowmode = oldSettings.bargrowmode;
    end
    if oldSettings.iconsizemode ~= nil then
        settings.iconsizemode = oldSettings.iconsizemode;
    end
    if oldSettings.firstload ~= nil then
        settings.firstload = oldSettings.firstload;
    end


end


function MainWindow:GetAnchorPosition()
    local x = Turbine.UI.Display:GetWidth()*settings.mainwindow.position.left;
    local y = Turbine.UI.Display:GetHeight()*settings.mainwindow.position.top;

    x = math.max(30, math.min(x,Turbine.UI.Display:GetWidth()-self.baseWidth-10));
    y = math.max(self.baseHeight, math.min(y,Turbine.UI.Display:GetHeight()-self.baseHeight-10));

    return x, y;
end

function MainWindow:ShowOptions()
    if self.options == nil then
        self.options = OptionsWindow();
    end
    self.options:SetVisible(true);
end

function MainWindow:Hide()
    self.shown = false;
    self:CheckVisible();
end

function MainWindow:Reset()
    mainWindow.activeFrame:Reset();
end

function MainWindow:CheckVisible()
    if (not self.hudvisible) or (not self.dragable and not self.shown) then
        self:SetVisible(false);
        self.activeFrame:Show(false);
    else
        self:SetVisible(self.shown or self.dragable);
        self.activeFrame:Show(self.shown and not self.dragable);
    end
end

function MainWindow:MouseDown( args )
    if (self.dragable and args.Button == Turbine.UI.MouseButton.Left) then
        self.dragStartX = args.X;
        self.dragStartY = args.Y;
        self.dragging = true;
    end
end

function MainWindow:MouseMove( args )
    local left, top = self:GetPosition();

    if (self.dragging) then
        local newLeft = left + ( args.X - self.dragStartX );
        local newTop = top + ( args.Y - self.dragStartY );

        newTop = math.max(self.baseHeight, math.min(newTop, Turbine.UI.Display:GetHeight() - self.baseHeight));
        newLeft = math.max(30, math.min(newLeft, Turbine.UI.Display:GetWidth() - self.baseWidth));

        self:SetPosition(newLeft,newTop);
    end
end

function MainWindow:MouseUp( args )
    if (self.dragging and args.Button==Turbine.UI.MouseButton.Left) then
        self.dragging = false;

        local newLeft, newTop = self:GetPosition();

        settings.mainwindow.position.left = newLeft/Turbine.UI.Display:GetWidth();
        settings.mainwindow.position.top = newTop/Turbine.UI.Display:GetHeight();

    end
end

function MainWindow:CreateHoverBox( )
    if self.hoverGem == nil then
        self.hoverGem = Turbine.UI.Control();
        self.hoverGem:SetParent(self);
        self.hoverGem:SetPosition(0, 0);
        self.hoverGem:SetBackground(C_MOVEGEM_PATH);
        self.hoverGem:SetSize(21,20);
        self.hoverGem:SetMouseVisible(false);
        self.hoverGem:SetOpacity(1);
        self.hoverGem:SetVisible(true);
        self.hoverGem:SetZOrder(self:GetZOrder()+3);

        self.hoverBar = Turbine.UI.Control();
        self.hoverBar:SetParent(self);
        self.hoverBar:SetPosition(21, 0);
        self.hoverBar:SetSize(1,20);
        self.hoverBar:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
        self.hoverBar:SetBackColorBlendMode(Turbine.UI.BlendMode.Multiply);
        self.hoverBar:SetBackColor(Turbine.UI.Color(0, 0, 0, 0 ));
        self.hoverBar:SetBackground(C_MOVEBAR_PATH);
        self.hoverBar:SetStretchMode(1);
        self.hoverBar:SetSize(self:GetWidth()-20,20);
        self.hoverBar:SetMouseVisible(false);
        self.hoverBar:SetOpacity(1);
        self.hoverBar:SetVisible(true);
        self.hoverBar:SetZOrder(self:GetZOrder()+2);

        self.hoverTitle = Turbine.UI.Label();
        self.hoverTitle:SetParent(self);
        self.hoverTitle:SetForeColor(Turbine.UI.Color( 1, 1, 1, 1 ) );
        self.hoverTitle:SetOutlineColor(Turbine.UI.Color( 1, 0, 0, 0 ) );
        self.hoverTitle:SetBackColor(Turbine.UI.Color(0, 0, 0, 0 ));
        self.hoverTitle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
        self.hoverTitle:SetBackColorBlendMode(Turbine.UI.BlendMode.Multiply);
        self.hoverTitle:SetFont(Turbine.UI.Lotro.Font.Verdana12);
        self.hoverTitle:SetFontStyle(Turbine.UI.FontStyle.Outline);
        self.hoverTitle:SetPosition(23, 2);
        self.hoverTitle:SetSize(70,20);
        self.hoverTitle:SetText(C_PLUGIN);
        self.hoverTitle:SetMouseVisible(false);
        self.hoverTitle:SetOpacity(1);
        self.hoverTitle:SetVisible(true);
        self.hoverTitle:SetZOrder(self:GetZOrder()+3);
    end
end

function MainWindow:DestroyHoverBox( )
    if self.hoverGem ~= nil then
        self.hoverGem:SetVisible(false);
        self.hoverTitle:SetVisible(false);
        self.hoverBar:SetVisible(false);
        self.hoverGem = nil;
        self.hoverTitle = nil;
        self.hoverBar = nil;
    end
end

function MainWindow:ToggleRepositioning()
    self.dragable = not self.dragable;
    if(self.dragable) then
        local x,y = self:GetAnchorPosition();
        self:SetSize(self.baseWidth, self.baseHeight);
        self:SetPosition(x,y);
        self:CreateHoverBox();
    else
        self:DestroyHoverBox();
        self.activeFrame:RecalcPositionFromBarUpdate();
    end
    self:CheckVisible();
end

function MainWindow:AddSync(member, song, instrument, length)
    self.shown = true;
    self.activeFrame:AddSync(member, song, instrument, length);
    self:CheckVisible();
end


function MainWindow:LostSync(member)
    if self.activeFrame:GetNumMembers() > 0 then
        self.shown = true;
    end
    self.activeFrame:LostSync(member)
    self:CheckVisible();
end

function MainWindow:OnMusicStarting()
    self.shown = true;
    self.activeFrame:OnMusicStarting()
    self:CheckVisible();
end
