import "Turbine.Gameplay";

local locale = nil;
local player = nil;
local parseFunction = nil;

function InitLocaleStrings()
    locale = "en";
    parseFunction = ParseChatEN;
    if Turbine.Shell.IsCommand("hilfe") then
        locale = "de";
        parseFunction = ParseChatDE;
    elseif Turbine.Shell.IsCommand("aide") then
        locale = "fr";
        parseFunction = ParseChatFR;
    end
end

function IsInstrument(teststring)
    if string.find(teststring, "lute") then
        return true;
    elseif string.find(teststring, "bell") then
        return true;
    elseif string.find(teststring, "harp") then
        return true;
    elseif string.find(teststring, "drum") then
        return true;
    elseif string.find(teststring, "horn") then
        return true;
    elseif string.find(teststring, "theorbo") then
        return true;
    elseif string.find(teststring, "bagpipe") then
        return true;
    elseif string.find(teststring, "clarinet") then
        return true;
    elseif string.find(teststring, "pibgorn") then
        return true;
    end
    return false;
end

function CleanString(s)
    s=s:gsub("[%-~]", "");
    return s:match'^()%s*$' and '' or s:match'^%s*(.*%S)';
end

function ParseSyncEvent(window, member, song)
    if member == nil then
        if player == nil then
            local localPlayer = Turbine.Gameplay.LocalPlayer.GetInstance();
            if localPlayer ~= nil then
                player = localPlayer:GetName();
            else
                player = "";
            end
        end
        member = player;
    end

    -- Remove all bracketed elements.
    local length = nil;
    local base = nil;
    local instrument = nil;

    for sub in string.gmatch(song, "[%(%[](%d+%p%d+)[%)%]]") do
        --Turbine.Shell.WriteLine("SubA '"..sub.."'\n");
        if length == nil then
            --Turbine.Shell.WriteLine("LenA '"..sub.."'\n");
            length = sub;
        end
    end

    if length then
        song = string.gsub(song, "[%(%[]%d+%p%d+[%)%]]", "~");
    else
        for sub in string.gmatch(song, "(%d+%p%d+)") do
            --Turbine.Shell.WriteLine("SubB '"..sub.."'\n");
            if length == nil then
                --Turbine.Shell.WriteLine("LenB '"..sub.."'\n");
                length = sub;
            end
        end
        if length then
            song = string.gsub(song, "%d+%p%d+", "~");
        end
    end



    for sub in string.gmatch(song, "%((.+)%)") do
        --Turbine.Shell.WriteLine("SubC '"..sub.."'\n");
        if instrument == nil then
            if IsInstrument(string.lower(sub)) then
                --Turbine.Shell.WriteLine("InsC '"..sub.."'\n");
                instrument = sub;
            end
        end
    end

    for sub in string.gmatch(song, "%[(.+)%]") do
        --Turbine.Shell.WriteLine("SubD '"..sub.."'\n");
        if instrument == nil then
            if IsInstrument(string.lower(sub)) then
                --Turbine.Shell.WriteLine("InsD '"..sub.."'\n");
                instrument = sub;
            end
        end
    end

    song = string.gsub(song, "[%(%[].+[%)%]]", "~");

    local firstMarker = string.find(song, "~");
    if firstMarker then
        --Turbine.Shell.WriteLine("FirstA "..firstMarker.." song '"..song.."'\n");
        if firstMarker > 5 or firstMarker == string.len(song) then
            base = string.sub(song, 1, firstMarker-1);
        else
            firstMarker = string.find(song, "~", firstMarker+1);
            if firstMarker then
                --Turbine.Shell.WriteLine("FirstB "..firstMarker.." song '"..song.."'\n");
                if firstMarker > 5 or firstMarker == string.len(song) then
                    base = string.sub(song, 1, firstMarker-1);
                else
                    base = song;
                end
            else
                base = song;
            end
        end
    else
        base = song;
    end

    -- If no instrument determined, walk back along the string
    -- marker by marker until we find a marked element containing
    -- an instrument name.
    if instrument == nil then
        local matchLast = string.match(song, ".*~(.+)$");
        while matchLast do
            ----Turbine.Shell.WriteLine("last='"..matchLast.."'\n");
            if IsInstrument(string.lower(matchLast)) then
                instrument = matchLast;
                base = string.sub(song, 1, string.len(song) - string.len(matchLast));
                matchLast = nil;
            else
                song = string.sub(song, 1, string.len(song) - string.len(matchLast));
                matchLast = string.match(song, ".*~(.+)$");
            end
        end
    end

    -- If no instrument determined, walk back along the string
    -- marker by marker until we find a marked element containing
    -- an instrument name.
    if instrument == nil then
        local matchLast = string.match(song, ".*%-(.+)$");
        while matchLast do
            ----Turbine.Shell.WriteLine("last='"..matchLast.."'\n");
            if IsInstrument(string.lower(matchLast)) then
                instrument = matchLast;
                base = string.sub(song, 1, string.len(song) - string.len(matchLast));
                matchLast = nil;
            else
                song = string.sub(song, 1, string.len(song) - string.len(matchLast));
                matchLast = string.match(song, ".*%-(.+)$");
            end
        end
    end

    -- Still no instrument known, so try walking back alone the base name
    -- until we find the last non-digit word and see if that's an
    -- instrument name. If it is, use it and anything that followed it as the instrument.
    if instrument == nil then
        local tail = ""
        local matchLast = string.match(song, ".*%s(.+)$");
        while matchLast do
            ----Turbine.Shell.WriteLine("lastw='"..matchLast.."'\n");
            if IsInstrument(string.lower(matchLast)) then
                instrument = matchLast..tail;
                base = string.sub(song, 1, string.len(song) - string.len(matchLast));
                matchLast = nil;
            else
                tail = tail..matchLast;
                song = string.sub(song, 1, string.len(song) - string.len(matchLast));
                matchLast = string.match(song, ".*%s(.+)$");
            end
        end
    end

    if base == nil then
        base = "";
    end

    if instrument == nil then
        instrument = "";
    end

    instrument = CleanString(instrument);
    base = CleanString(base);

    window:AddSync(member, base, instrument, GetSongLength(length));
end

function GetSongLength(length)
    local songLength = 0;
    if length ~= nil then
        local mins, seconds = string.match(length, "(%d+)%p(%d+)");
        if mins then
            songLength = songLength + (tonumber(mins)*60);
        end
        if seconds then
            songLength = songLength + tonumber(seconds);
        end
    end
    return songLength;
end

function ParseChat(window, message)
    if parseFunction ~= nil then
        parseFunction(window, message);
    end
end

function ParseChatEN(window, message)
    if string.find(message, "Synchronized playing is about to begin.") then
        window:OnMusicStarting()
    elseif string.find(message, " ready to play ") ~= nil then
        local member, song = string.match(message, "(.+) ready to play \"(.+)%.\"");
        if song ~= nil then
            member = string.match(member, "(.+) is");
            ParseSyncEvent(window, member, song);
        end
    else
        local member = string.match(message, "(.+) has left your");
        if member ~= nil then
            window:LostSync(member);
        else
            member = string.match(message, "(.+) has gone link dead from your");
            if member ~= nil then
                window:LostSync(member);
            end
        end
    end
end

function ParseChatFR(window, message)
    if string.find(message, "La lecture synchronisée va commencer.") then
        window:OnMusicStarting()
    elseif string.find(message, " jouer ") ~= nil then
        local song = string.match(message, "Vous allez jouer \"(.+)%.\"");
        if song ~= nil then
            ParseSyncEvent(window, nil, song);
        else
            local member, song = string.match(message, "(.+) va jouer \"(.+)%.\"");
            if song ~= nil and member ~= nil then
                ParseSyncEvent(window, member, song);
            end
        end
    elseif string.find(message, " votre ") ~= nil then
        local member = string.match(message, "(.+), membre de votre .+, est apparemment Hors connexion%.");
        if member ~= nil then
            window:LostSync(member);
        else
            member = string.match(message, "(.+) a quitté votre .+%.");
            if member ~= nil then
                window:LostSync(member);
            end
        end
    end
end

function ParseChatDE(window, message)
    if string.find(message, "Synchronisiertes Spiel beginnt bald.") then
        window:OnMusicStarting()
    elseif string.find(message, " kann nun gespielt werden.") ~= nil then
        local song = string.match(message, "\"(.+)\" kann nun gespielt werden%.");
        if song ~= nil then
            ParseSyncEvent(window, nil, song);
        end
    elseif string.find(message, " zu spielen.") ~= nil then
        local member, song = string.match(message, "(.+) ist bereit \"(.+)\" zu spielen%.");
        if song ~= nil and member ~= nil then
            ParseSyncEvent(window, member, song);
        end
    elseif string.find(message, " Gruppe von Gefährten ") ~= nil then
        local member = string.match(message, "(.+) hat Eure Gruppe von Gefährten verlassen%.");
        if member ~= nil then
            window:LostSync(member);
        else
            member = string.match(message, "(.+) hat die Verbindung zu Eurer Gruppe von Gefährten verloren%.");
            if member ~= nil then
                window:LostSync(member);
            end
        end
    end
end