import "Turbine.Gameplay";

local C_BARGOOD_PATH = "Serbryt/BandAide/Resources/BarGreen.tga"
local C_BARBAD_PATH = "Serbryt/BandAide/Resources/BarRed.tga"
local C_DRUM_PATH = "Serbryt/BandAide/Resources/i_drum"
local C_HORN_PATH = "Serbryt/BandAide/Resources/i_horn"
local C_LUTE_PATH = "Serbryt/BandAide/Resources/i_lute"
local C_FLUTE_PATH = "Serbryt/BandAide/Resources/i_flute"
local C_CLARINET_PATH = "Serbryt/BandAide/Resources/i_clarinet"
local C_BAGPIPE_PATH = "Serbryt/BandAide/Resources/i_bagpipe"
local C_COWBELL_PATH = "Serbryt/BandAide/Resources/i_cowbell"
local C_HARP_PATH = "Serbryt/BandAide/Resources/i_harp"
local C_THEORBO_PATH = "Serbryt/BandAide/Resources/i_theorbo"
local C_PIBGORN_PATH = "Serbryt/BandAide/Resources/i_pibgorn"
local C_UNK_INST_PATH = "Serbryt/BandAide/Resources/i_unknown"

local C_BARTEXT_BACKCOLOR = Turbine.UI.Color(0, 0, 0, 0 )
local C_BARTEXT_FORECOLOR = Turbine.UI.Color( 1, 1, 1, 1 )
local C_BARTEXT_ALTCOLOR = Turbine.UI.Color( 1, 0.9, 0.9, 0.2 )
local C_BARTEXT_OUTCOLOR = Turbine.UI.Color( 1, 0, 0, 0 )
local C_BARTEXT_FONTSTYLE = Turbine.UI.FontStyle.Outline
local C_BARTEXT_FONT = Turbine.UI.Lotro.Font.Verdana12
local C_BAR_NAMEWIDTH = 75
local C_BAR_SONGWIDTH = 140

PlayerBar = class(Turbine.UI.Control);

function GetIconFile(basepath)
    if GetIconSize() == 16 then
        return basepath..".tga";
    end
    return basepath.."_l.tga";
end

function PlayerBar:Constructor(frame, x, y, w, h, playerName, song, instrument, length)
    Turbine.UI.Control.Constructor(self);

    self.playerName = Capitalize(playerName);
    self.song = song;
    self.instrument = string.lower(instrument);
    self.icode = 0;
    if length then
        self.length = length;
    else
        self.length = 0;
    end
    self.songMismatch = false;
    self.lenMismatch = false;

    self:SetParent(frame);
    self:SetPosition(x+GetIconSize()+2,y);
    self:SetSize(w-GetIconSize(),h);
    self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self:SetBackground(C_BARGOOD_PATH);
    self:SetBackColor(Turbine.UI.Color(0, 0, 0, 0 ));
    self:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
    self:SetMouseVisible(false);
    self:SetOpacity(0.4);

    self.playerText = Turbine.UI.Label();
    self.playerText:SetBackColor(C_BARTEXT_BACKCOLOR);
    self.playerText:SetForeColor(C_BARTEXT_FORECOLOR);
    self.playerText:SetOutlineColor(C_BARTEXT_OUTCOLOR);
    self.playerText:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
    self.playerText:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
    self.playerText:SetFontStyle(C_BARTEXT_FONTSTYLE);
    self.playerText:SetFont(C_BARTEXT_FONT);
    self.playerText:SetParent(self);
    self.playerText:SetPosition(8,1);
    self.playerText:SetSize(C_BAR_NAMEWIDTH+C_BAR_SONGWIDTH,12);
    self.playerText:SetMouseVisible(false);

    self.playerText2 = Turbine.UI.Label();
    self.playerText2:SetBackColor(C_BARTEXT_BACKCOLOR);
    self.playerText2:SetForeColor(C_BARTEXT_ALTCOLOR);
    self.playerText2:SetOutlineColor(C_BARTEXT_OUTCOLOR);
    self.playerText2:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
    self.playerText2:SetBackColorBlendMode( Turbine.UI.BlendMode.AlphaBlend );
    self.playerText2:SetFontStyle(C_BARTEXT_FONTSTYLE);
    self.playerText2:SetFont(C_BARTEXT_FONT);
    self.playerText2:SetParent(self);
    self.playerText2:SetPosition(24,17);
    self.playerText2:SetSize(C_BAR_NAMEWIDTH+C_BAR_SONGWIDTH,12);
    self.playerText2:SetMouseVisible(false);
    self.playerText2:SetOpacity(0.9);

    self.instrumentImg = Turbine.UI.Control();
    self.instrumentImg:SetParent(frame);
    self.instrumentImg:SetPosition(x, y);
    self.instrumentImg:SetSize(GetIconSize(), GetIconSize());
    self.instrumentImg:SetMouseVisible(false);

    self:UpdateInstrument();

    self:UpdateText();

    self:SetVisible(true);
    self.playerText:SetVisible(true);
    self.playerText2:SetVisible(GetIconSize() > 16);
    self.instrumentImg:SetVisible(true);
end

function PlayerBar:GetTotalWidth()
    return C_BAR_NAMEWIDTH + GetIconSize() + C_BAR_SONGWIDTH;
end

function PlayerBar:UpdateBar(song, instrument, length)
    local updateImg = (self.instrument ~= string.lower(instrument));
    self.instrument = string.lower(instrument);
    if updateImg then
        self:UpdateInstrument();
    end
    self.song = song;
    self.length = length;
    self:UpdateText();
end

function PlayerBar:UpdateText()
    if self.songMismatch == true then
        self.playerText:SetText(self.playerName.."  ("..Capitalize(self.song)..")");
        self:SetBackground(C_BARBAD_PATH);
    elseif self.lenMismatch == true then
        self.playerText:SetText(self.playerName.."  "..TimeString(self.length));
        self:SetBackground(C_BARBAD_PATH);
    else
        self.playerText:SetText(self.playerName);
        self:SetBackground(C_BARGOOD_PATH);
    end
    self:UpdateAltText();
end

function PlayerBar:UpdateInstrument()
    if self.instrument:find("flute") then
        self.instrumentImg:SetBackground(GetIconFile(C_FLUTE_PATH));
        self.icode = 51;
    elseif self.instrument:find("lute") then
        self.instrumentImg:SetBackground(GetIconFile(C_LUTE_PATH));
        self.icode = 11;
    elseif self.instrument:find("drum") then
        self.instrumentImg:SetBackground(GetIconFile(C_DRUM_PATH));
        self.icode = 1;
    elseif self.instrument:find("horn") then
        self.instrumentImg:SetBackground(GetIconFile(C_HORN_PATH));
        self.icode = 61;
    elseif self.instrument:find("clarinet") then
        self.instrumentImg:SetBackground(GetIconFile(C_CLARINET_PATH));
        self.icode = 71;
    elseif self.instrument:find("bagpipe") then
        self.instrumentImg:SetBackground(GetIconFile(C_BAGPIPE_PATH));
        self.icode = 41;
    elseif self.instrument:find("pibgorn") then
        self.instrumentImg:SetBackground(GetIconFile(C_PIBGORN_PATH));
        self.icode = 91;
    elseif self.instrument:find("bell") then
        self.instrumentImg:SetBackground(GetIconFile(C_COWBELL_PATH));
        self.icode = 81;
    elseif self.instrument:find("harp") then
        self.instrumentImg:SetBackground(GetIconFile(C_HARP_PATH));
        self.icode = 21;
    elseif self.instrument:find("theorbo") then
        self.instrumentImg:SetBackground(GetIconFile(C_THEORBO_PATH));
        self.icode = 31;
    else
        self.instrumentImg:SetBackground(GetIconFile(C_UNK_INST_PATH));
        self.icode = 101;
    end
    self:UpdateAltText();
end

function PlayerBar:UpdateAltText()
    if GetIconSize() > 16 then
        self.playerText2:SetText(Capitalize(self.instrument));
        self.playerText2:SetVisible(true);
    else
        self.playerText2:SetVisible(false);
    end
end

function PlayerBar:Reposition(x,y)
    self:SetPosition(x+GetIconSize()+2,y);
    self.instrumentImg:SetPosition(x,y);
    if self.instrumentImg:GetWidth() ~= GetIconSize() then
        self:SetHeight(GetIconSize());
        self.instrumentImg:SetSize(GetIconSize(), GetIconSize());
        self:UpdateInstrument();
    end
end

function PlayerBar:RemoveBar()
    self.playerText2:SetVisible(false);
    self.playerText:SetVisible(false);
    self.instrumentImg:SetVisible(false);
    self:SetVisible(false);
end

function PlayerBar:GetSong()
    return self.song;
end

function PlayerBar:GetInstrument()
    return self.instrument;
end

function PlayerBar:GetLength()
    return self.length;
end

function PlayerBar:IsSongMismatched(testSong, testLength)
    local songMismatch = (testSong ~= self.song);
    local diffLen = testLength - self.length;
    local lenMismatch = false;
    if (diffLen > 10) or (diffLen < -10) then
        lenMismatch = true;
    end
    local updtText = false;
    if songMismatch ~= self.songMismatch then
        self.songMismatch = songMismatch;
        updtText = true;
    end
    if lenMismatch ~= self.lenMismatch then
        self.lenMismatch = lenMismatch;
        updtText = true;
    end
    if updtText == true then
        self:UpdateText();
    end
    return self.songMismatch or self.lenMismatch;
end

