import "Turbine.UI";

local C_NUM_X_OFFSET = 30
local C_LINE_X_OFFSET = 55
local C_LINE_Y_OFFSET = 9

PlayerFrame = class(Turbine.UI.Control);

function PlayerFrame:Constructor(parent,x,y,w,h,background)
    Turbine.UI.Control.Constructor(self);

    self.lastSec = 0;
    self.songEnd = nil;
    self.curSong = nil;
    self.mismatch = false;

    self:SetBackColor(Turbine.UI.Color(0,0,0,0));
    self:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self:SetParent(parent);
    self:SetPosition(x,y);
    self:SetBackground(background);
    self:SetMouseVisible(false);
    self:SetSize(w,h);
    self:SetOpacity(0.5);
    self:SetZOrder(parent:GetZOrder()+1);

    self.contextMenu = Turbine.UI.ContextMenu();
    local menuItems = self.contextMenu:GetItems();

    local menuItem = Turbine.UI.MenuItem("Reset");
    menuItem.Click = function(sender, args) mainWindow:Reset(); end
    menuItems:Add(menuItem);

    menuItem = Turbine.UI.MenuItem("Hide");
    menuItem.Click = function(sender, args) mainWindow:Hide(); end
    menuItems:Add(menuItem);

    menuItem = Turbine.UI.MenuItem("Options");
    menuItem.Click = function(sender, args) mainWindow:ShowOptions(); end
    menuItems:Add(menuItem);

    self.title = Turbine.UI.Label();
    self.title:SetBackColor(self:GetBackColor());
    self.title:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.title:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.title:SetForeColor(Turbine.UI.Color( 1, 1, 1, 1 ) );
    self.title:SetOutlineColor(Turbine.UI.Color( 1, 0, 0, 0 ) );
    self.title:SetFontStyle(Turbine.UI.FontStyle.Outline );
    self.title:SetFont(Turbine.UI.Lotro.Font.Verdana12);
    self.title:SetParent(self);
    self.title:SetPosition(C_LINE_X_OFFSET,C_LINE_Y_OFFSET);
    self.title:SetSize(174,13);
    self.title:SetMouseVisible(false);
    self.title:SetZOrder(self:GetZOrder()+2);

    self.syncCount = Turbine.UI.Label();
    self.syncCount:SetBackColor(self:GetBackColor());
    self.syncCount:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.syncCount:SetForeColor(Turbine.UI.Color( 1, 1, 1, 1 ) );
    self.syncCount:SetOutlineColor(Turbine.UI.Color( 1, 0, 0, 0 ) );
    self.syncCount:SetFontStyle(Turbine.UI.FontStyle.Outline );
    self.syncCount:SetFont(Turbine.UI.Lotro.Font.TrajanPro26);
    self.syncCount:SetParent(self);
    self.syncCount:SetPosition(C_NUM_X_OFFSET,C_LINE_Y_OFFSET+5);
    self.syncCount:SetSize(40,28);
    self.syncCount:SetMouseVisible(false);
    self.syncCount:SetZOrder(self:GetZOrder()+1);

    self.syncBad = Turbine.UI.Label();
    self.syncBad:SetBackColor(self:GetBackColor());
    self.syncBad:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.syncBad:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.syncBad:SetForeColor(Turbine.UI.Color( 1, 0, 0, 0 ) );
    self.syncBad:SetOutlineColor(Turbine.UI.Color( 0, 0, 0, 0 ) );
    self.syncBad:SetFontStyle(Turbine.UI.FontStyle.Outline );
    self.syncBad:SetFont(Turbine.UI.Lotro.Font.Verdana14);
    self.syncBad:SetParent(self);
    self.syncBad:SetPosition(C_NUM_X_OFFSET-4,C_LINE_Y_OFFSET-5);
    self.syncBad:SetSize(28,28);
    self.syncBad:SetMouseVisible(false);
    self.syncBad:SetZOrder(self:GetZOrder()+1);
    self.syncBad:SetText("#");
    self.syncBad:SetVisible(false);

    self.playTitle = Turbine.UI.Label();
    self.playTitle:SetBackColor(self:GetBackColor());
    self.playTitle:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.playTitle:SetBackColorBlendMode(Turbine.UI.BlendMode.AlphaBlend);
    self.playTitle:SetForeColor(Turbine.UI.Color( 1, 1, 1, 1 ));
    self.playTitle:SetOutlineColor(Turbine.UI.Color( 1, 0, 0, 0 ));
    self.playTitle:SetFontStyle(Turbine.UI.FontStyle.Outline);
    self.playTitle:SetFont(Turbine.UI.Lotro.Font.Verdana12);
    self.playTitle:SetParent(self);
    self.playTitle:SetPosition(C_LINE_X_OFFSET,C_LINE_Y_OFFSET+16);
    self.playTitle:SetSize(175,13);
    self.playTitle:SetMouseVisible(false);
    self.playTitle:SetText("");
    self.playTitle:SetZOrder(self:GetZOrder()+1);

    self:UpdateTitle(0, nil, nil);
    self:UpdatePlayingTitle(nil,0);
    self:Show(true);

    self.Update = function()
        local clear = false;
        local now = Turbine.Engine:GetLocalTime();
        if self.curSong == nil then
            clear = true;
        elseif self.songEnd and now >= self.songEnd then
            clear = true;
        end

        if clear then
            self:StopPlaying();
            return;
        end

        if self.lastSec >= now then
            return;
        end

        self.lastSec = now;
        self.playTitle:SetText(TimeString(self.songEnd - now).." "..self.curSong);
    end
end

function PlayerFrame:MouseDown( args )
    if (self:IsVisible() == true and args.Button == Turbine.UI.MouseButton.Right) then
        self.contextMenu:ShowMenu();
    end
end

function PlayerFrame:StopPlaying()
    self.songEnd = nil;
    self.curSong = nil;
    self:SetWantsUpdates(false);
    if self.playTitle then
        self.playTitle:SetText("");
    end
end

function PlayerFrame:Show(show)
    self:SetVisible(show);
    self:SetMouseVisible(show);
    self.title:SetVisible(show);
    self.syncCount:SetVisible(show);
    self.playTitle:SetVisible(show);
    self.syncBad:SetVisible(self.mismatch and show);
end

function PlayerFrame:UpdateTitle(numMembers, mismatch, bestSong)
    self.syncCount:SetText(numMembers);
    self.mismatch = mismatch;
    self.syncBad:SetVisible(mismatch);
    if mismatch == true then
        self.syncCount:SetForeColor(Turbine.UI.Color( 1, 1, 0.2, 0.2 ) );
    else
        self.syncCount:SetForeColor(Turbine.UI.Color( 1, 1, 1, 1 ) );
    end

    if bestSong then
        self.title:SetText(Capitalize(bestSong));
    else
        self.title:SetText("");
    end
end

function PlayerFrame:UpdatePlayingTitle(curSong, songLength)
    if curSong == nil or string.len(curSong) == 0 then
        self:StopPlaying();
        return;
    end

    local now = Turbine.Engine:GetLocalTime();
    self.curSong = curSong;
    if songLength > 0 then
        self.lastSec = now - 1;
        self.songEnd = now + songLength;
    else
        self.songEnd = now + 30;
        self.lastSec = now + 30;
        self.playTitle:SetText("Playing \""..curSong.."\"");
    end
    self:SetWantsUpdates(true);
end
