function WriteLine(str)
    Turbine.Shell.WriteLine(str);
end

function dump(arg)
    if type(arg) == "table" then
        for k,v in pairs(arg) do
            if type(v) == "table" then
                WriteLine(k .. ": (table)");
                dump(v);
                WriteLine("(end table " .. k .. ")");
            elseif (type(v) == "number" or type(v) == "string") then
                WriteLine(k .. ": " .. v);
            else
                WriteLine(k .. ": (" .. type(v) .. ")");
            end
        end
    else
        WriteLine(arg);
    end
end

function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end

function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

function convSafe(obj)
    local lookup = {}
    local function _copy(obj)
        function _convSafe(obj)
            if type(obj)=="number" then return(tostring(obj)) end
            if type(obj)~="string" then return(obj) end
            local res = ""
            for i,v in ipairs({obj:byte(1,-1)}) do
                if (v < 32) or (v > 125) then res = res .. "#" .. tonumber(v) .. "#"
                else
                    res = res .. string.char(v)
                    if v == 35 then res = res .. "#" end
                end
            end
            if res:byte(1,1) == 35 then return("#" .. res) end
            if (tonumber(res)~= nil) then return("#" .. res) end
            return res
        end
        if type(obj) ~= "table" then return _convSafe(obj) elseif lookup[obj] then return lookup[obj] end
        local newt = {}
        lookup[obj] = newt
        for i, v in pairs(obj) do newt[_copy(i)] = _copy(v) end
        return setmetatable(newt, getmetatable(obj))
    end
    return _copy(obj)
end

function convBack(obj)
    local lookup = {}
    local function _copy(obj)
        local function _convBack(obj)
            if obj == "" then return "" end
            local num = tonumber(obj)
            if num ~= nil then return num end
            if type(obj) ~= "string" then return obj end
            local res = ""
            if obj:byte(1,1) == 35 then obj = string.sub(obj,2,-1) end
            while string.len(obj)~=0 do
                if      obj:byte(1,1) ~= 35 then res = res .. string.char(obj:byte(1,1)); obj = string.sub(obj,2,-1)
                elseif obj:byte(2,2) == 35 then  res = res .. "#"; obj = string.sub(obj,3,-1)
                else
                    obj = string.sub(obj,2,-1)
                    local k = 1;
                    while (obj:byte(k,k) ~= nil) and (obj:byte(k,k) ~= 35) do k = k + 1 end
                    if obj:byte(k,k) == nil then error("convBack : parse error 1 !") end
                    local v = tonumber(string.sub(obj,1,k-1))
                    if v == nil then error("convBack : parse error 2 !") end
                    res = res .. string.char(v)
                    obj = string.sub(obj,k+1,-1)
                end
            end
            return res;
        end
        if type(obj) ~= "table" then return _convBack(obj) elseif lookup[obj] then return lookup[obj] end
        local newt = {}
        lookup[obj] = newt
        for i, v in pairs(obj) do newt[_copy(i)] = _copy(v) end
        return setmetatable(newt, getmetatable(obj))
    end
    return _copy(obj)
end

-- Replacement for Turbine.PluginData.Load
function PatchDataLoad(a,b,c)
    return convBack(Turbine.PluginData.Load(a,b,c))
end

-- Replacement for Turbine.PluginData.Save
function PatchDataSave(a,b,c,d)
    return Turbine.PluginData.Save(a,b,convSafe(c),d)
end

function spairs(t, order)
    local keys = {}
    for k in pairs(t) do keys[#keys+1] = k end

    if order then
        table.sort(keys, function(a,b) return order(t, a, b) end)
    else
        table.sort(keys)
    end

    local i = 0
    return function()
        i = i + 1
        if keys[i] then
            return keys[i], t[keys[i]]
        end
    end
end


--import "Turbine.Utils";

Table = class();

local function DumpTable( t, indentation, seen )
    if ( t == nil ) then
        Turbine.Shell.WriteLine( indentation .. "(nil)" );
        return;
    end

    seen[t] = true

    local s= {}
    local n = 0

    for k in pairs( t ) do
        n = n + 1
        s[n] = k
    end

    table.sort( s, function( a, b ) return tostring( a ) < tostring( b ) end )

    for k,v in pairs(s) do
        Turbine.Shell.WriteLine( indentation .. tostring( v ) .. ": " .. tostring( t[v] ) );
        v = t[v]

        if type( v ) == "table" and not seen[v] then
            DumpTable( v, indentation .. "  ", seen )
        end
    end
end

Table.Dump = function( t, indentation )
    local seen = { }
    DumpTable( t, indentation or "  ", seen );
end

Table.Copy=function(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
--        return setmetatable(new_table, getmetatable(object))
        return setmetatable(new_table, _copy(getmetatable(object)))
    end
    return _copy(object)
end

function Capitalize(astring)
    return string.gsub(astring, "(%a)([%w_']*)", function ( first, rest ) return first:upper()..rest:lower() end)
end

function TimeString(seconds)
    local minutes = math.floor(seconds/60);
    return string.format("[%d:%02d]", minutes, seconds - minutes*60);
end
